#!/usr/bin/env python

from unittest import TestCase

from before_after import before, after, before_after
from before_after.tests import test_functions


class TestBeforeAfter(TestCase):
    def setUp(self):
        test_functions.reset_test_list()
        super(TestBeforeAfter, self).setUp()

    def test_before(self):
        def before_fn(*a):
            test_functions.test_list.append(1)

        with before('before_after.tests.test_functions.sample_fn', before_fn):
            test_functions.sample_fn(2)

        self.assertEqual(test_functions.test_list, [1, 2])

    def test_after(self):
        def after_fn(*a):
            test_functions.test_list.append(2)

        with after('before_after.tests.test_functions.sample_fn', after_fn):
            test_functions.sample_fn(1)

        self.assertEqual(test_functions.test_list, [1, 2])

    def test_before_and_after(self):
        def before_fn(*a):
            test_functions.test_list.append(1)

        def after_fn(*a):
            test_functions.test_list.append(3)

        with before_after(
                'before_after.tests.test_functions.sample_fn',
                before_fn=before_fn, after_fn=after_fn):
            test_functions.sample_fn(2)

        self.assertEqual(test_functions.test_list, [1, 2, 3])

    def test_before_once(self):
        def before_fn(*a):
            test_functions.test_list.append(1)

        with before(
                'before_after.tests.test_functions.sample_fn',
                before_fn, once=True):
            test_functions.sample_fn(2)
            test_functions.sample_fn(3)

        self.assertEqual(test_functions.test_list, [1, 2, 3])

    def test_after_once(self):
        def after_fn(*a):
            test_functions.test_list.append(2)

        with after(
                'before_after.tests.test_functions.sample_fn',
                after_fn, once=True):
            test_functions.sample_fn(1)
            test_functions.sample_fn(3)

        self.assertEqual(test_functions.test_list, [1, 2, 3])

    def test_before_and_after_once(self):
        def before_fn(*a):
            test_functions.test_list.append(1)

        def after_fn(*a):
            test_functions.test_list.append(3)

        with before_after(
                'before_after.tests.test_functions.sample_fn',
                before_fn=before_fn, after_fn=after_fn, once=True):
            test_functions.sample_fn(2)
            test_functions.sample_fn(4)

        self.assertEqual(test_functions.test_list, [1, 2, 3, 4])



"""Package for before_after."""

__project__ = 'before_after'
__version__ = '0.1.3'

VERSION = __project__ + '-' + __version__

PYTHON_VERSION = 2, 7

import sys
if not sys.version_info >= PYTHON_VERSION:  # pragma: no cover (manual test)
    exit("Python {}.{}+ is required.".format(*PYTHON_VERSION))


from contextlib import contextmanager
from functools import wraps


def before(target, fn, **kwargs):
    return before_after(target, before_fn=fn, **kwargs)


def after(target, fn, **kwargs):
    return before_after(target, after_fn=fn, **kwargs)


@contextmanager
def before_after(
        target, before_fn=None, after_fn=None, once=True, **kwargs):
    def before_after_wrap(fn):
        called = []

        @wraps(fn)
        def inner(*a, **k):
            # If once is True, then don't call if this function has already
            # been called
            if once:
                if called:
                    return fn(*a, **k)
                else:
                    # Hack for lack of nonlocal keyword in Python 2: append to
                    # list to maked called truthy
                    called.append(True)

            if before_fn:
                before_fn(*a, **k)
            ret = fn(*a, **k)
            if after_fn:
                after_fn(*a, **k)
            return ret
        return inner

    from mock import patch

    patcher = patch(target, **kwargs)
    original, _ = patcher.get_original()
    with patcher as mock_fn:
        mock_fn.side_effect = before_after_wrap(original)
        yield



#!/usr/bin/env python

"""Setup script for before_after."""

import setuptools

from before_after import __project__, __version__


# Convert README.md from Markdown to rst if possible
try:
    from pypandoc import convert
except ImportError:
    read_md = lambda f: open(f, 'r').read()
else:
    read_md = lambda f: convert(f, 'rst')

README = read_md('README.md')

setuptools.setup(
    name=__project__,
    version=__version__,

    description="before_after provides utilities for testing race conditions",
    url='https://github.com/c-oreills/before_after',
    download_url='https://github.com/c-oreills/before_after/tarball/{v}'.
        format(v=__version__),
    author="Christy O'Reilly",
    author_email='christy@oreills.co.uk',

    packages=setuptools.find_packages(),

    entry_points={'console_scripts': []},

    long_description=README,
    license='GPLv2',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
    ],
    keywords = ['testing', 'race conditions'],

    install_requires=['mock==1.0.1'],
)
